#include "bezier.h"

/****************************************/
/* Define lights.                       */
/****************************************/
void SET_LIGHTS(void) {
GLfloat MatAmbient[]     = {1.0, 0.9, 0.2, 1.0};

GLfloat MatDiffuse[]     = {1.0, 0.9, 0.3, 1.0};
GLfloat Light1Diffuse[]  = {1.0, 1.0, 1.0, 1.0};
GLfloat Light1Position[] = {100.0, 100.0,  100.0, 1.0};

GLfloat MatShininess[]   = {4.0};
GLfloat MatSpecular[]    = {0.1, 0.1, 1.0, 1.0};
GLfloat Light2Specular[] = {1.0, 1.0, 1.0, 1.0};
GLfloat Light2Position[] = {0.0, 500.0, 10.0, 1.0};

   glLoadIdentity();
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
   glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_TRUE);
   // Ambient "light source".
   glMaterialfv(GL_FRONT, GL_AMBIENT, MatAmbient);
   // Diffuse light source.
   glMaterialfv(GL_FRONT, GL_DIFFUSE, MatDiffuse);
   glLightfv(GL_LIGHT1, GL_DIFFUSE,  Light1Diffuse);
   glLightfv(GL_LIGHT1, GL_POSITION, Light1Position);
   // Spectral light source.
   glMaterialfv(GL_FRONT, GL_SPECULAR, MatSpecular);
   glMaterialfv(GL_FRONT, GL_SHININESS, MatShininess);
   glLightfv(GL_LIGHT2, GL_SPECULAR, Light2Specular);
   glLightfv(GL_LIGHT2, GL_POSITION, Light2Position);
}

/****************************************/
/* Draw BEZIER TUBE in window.          */
/****************************************/
void BEZIER_TUBE(void) {
int    p, d, v;
int    x, y, z;
int    vertices = maxBalls-1;
int    points[maxBalls][3];
double k, p1, p2, p3, p4;
   // Draw end & control points.
   bezrFlag = 0;
   glLoadIdentity();  
   glPointSize(4);
   glColor3f(1.0, 0.0, 1.0);
   glBegin(GL_POINTS);
     if (cPoints[0][3] == 1) {
        glVertex3f(cPoints[0][0],cPoints[0][1],cPoints[0][2]);
        bezrFlag++;
     }
     if (cPoints[1][3] == 1) {
        glVertex3f(cPoints[1][0],cPoints[1][1],cPoints[1][2]);
        bezrFlag++;
     }
     if (cPoints[2][3] == 1) {
        glVertex3f(cPoints[2][0],cPoints[2][1],cPoints[2][2]);
        bezrFlag++;
     }
     if (cPoints[3][3] == 1) {
        glVertex3f(cPoints[3][0],cPoints[3][1],cPoints[3][2]);
        bezrFlag++;
     }
   glEnd();
   glPointSize(1);
   if (bezrFlag < 4) return;
   // Define curve.
   for (d = 0; d < 3; d++) {
       for (v = 0; v <= vertices; v++) {
           k  = (double)v / (double)vertices;
           p1 = cPoints[0][d]*(1.0-k)*(1.0-k)*(1.0-k);
           p2 = cPoints[1][d]*(1.0-k)*(1.0-k)*3*k;
           p3 = cPoints[2][d]*(1.0-k)*3*k*k;
           p4 = cPoints[3][d]*k*k*k;
           points[v][d] = (int)floor(p1+p2+p3+p4 + 0.5);
       }
   }
   // Draw curve points.
   glPointSize(1);
   glColor3f(1.0, 1.0, 1.0);
   for (v = 0; v < vertices; v++) {
       glBegin(GL_POINTS);
         glVertex3d(points[v][0], points[v][1], points[v][2]);
       glEnd();
   }
   glPointSize(1);
   // Draw arm.
   glLoadIdentity();  
   glTranslated(points[0][0], points[0][1], points[0][2]);
   if (armSolid == 1) {
      SET_LIGHTS();
      glEnable(GL_LIGHTING);
      glShadeModel(GL_FLAT);
      glEnable(GL_AUTO_NORMAL);
      if (light1 == 1) glEnable(GL_LIGHT1);
      if (light2 == 1) glEnable(GL_LIGHT2);
      glCullFace(GL_BACK);
      glEnable(GL_CULL_FACE);
      glEnable(GL_DEPTH_TEST);
      glClear(GL_DEPTH_BUFFER_BIT);
      glutSolidSphere((double)ballSize, 20, 20);
   }
   if (armWire == 1) glutWireSphere((double)ballSize, 20, 20);
   for (v = 1; v <= vertices; v++) {
       p1 = (x-points[v][0])*(x-points[v][0]);
       p2 = (y-points[v][1])*(y-points[v][1]);
       p3 = (z-points[v][2])*(z-points[v][2]);
       d  = (int)floor(sqrt(p1+p2+p3)+0.5);
       if ((2*ballSize - d) < overlap) {
          glLoadIdentity();  
          glTranslated(points[v][0], points[v][1], points[v][2]);
          if (armWire  == 1) glutWireSphere((double)ballSize, 20, 20);
          if (armSolid == 1) glutSolidSphere((double)ballSize, 20, 20);
          x = points[v][0];
          y = points[v][1];
          z = points[v][2];
       }
   }
   glLoadIdentity();  
   glTranslated(points[vertices][0], points[vertices][1], points[vertices][2]);
   if (armWire  == 1) glutWireSphere((double)ballSize, 20, 20);
   if (armSolid == 1) glutSolidSphere((double)ballSize, 20, 20);
   glDisable(GL_LIGHTING);
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_CULL_FACE);
}

/****************************************/
/* Displays object in TARGET window.    */
/****************************************/
void TARGET_DISPLAY() {
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   BEZIER_TUBE();
   glLoadIdentity();
   glutSwapBuffers();
}

/****************************************/
/* Idle function.                       */
/****************************************/
void ARM_MOTION(void) {
   if (bezrFlag < 4) return;
   system("sleep 0.1");
   moveSize++;
   if (moveE1 == 1) {
      cPoints[0][0] += (int)(15.0*cos((double)moveSize));
      cPoints[0][1] += (int)(15.0*sin((double)moveSize));
      cPoints[0][2] += (int)(30.0*cos((double)moveSize/30.0));
   }
   if (moveC1 == 1) {
      cPoints[1][0] += (int)(15.0*cos((double)moveSize));
      cPoints[1][1] += (int)(50.0*sin((double)moveSize));
//      cPoints[1][2] += (int)(30.0*sin((double)moveSize/30.0));
   }
   if (moveE2 == 1) {
      cPoints[3][0] += (int)(15.0*sin((double)moveSize));
      cPoints[3][1] += (int)(50.0*cos((double)moveSize));
//      cPoints[3][2] += (int)(30.0*cos((double)moveSize/30.0));
   }
   if (moveC2 == 1) {
      cPoints[2][0] += (int)(15.0*sin((double)moveSize));
      cPoints[2][1] += (int)(50.0*sin((double)moveSize));
      cPoints[2][2] += (int)(30.0*sin((double)moveSize/30.0));
   }
   TARGET_WINDOW.CLEAR();
   TARGET_DISPLAY();
}

/****************************************/
/* Main window display.                 */
/****************************************/
void MAIN_WINDOW(void) {
   glDrawBuffer(GL_BACK);
   glPolygonMode(GL_FRONT, GL_FILL);
   glColor3f(0.6, 0.6, 0.6);
   glBegin(GL_QUADS);
     glVertex2i(       0,         0);
     glVertex2i(winWidth,         0);
     glVertex2i(winWidth, winHeight);
     glVertex2i(       0, winHeight);
   glEnd();
   END1_BUTTON.DRAW();
   MVE1_BUTTON.DRAW();
   CNT1_BUTTON.DRAW();
   MVC1_BUTTON.DRAW();
   END2_BUTTON.DRAW();
   MVE2_BUTTON.DRAW();
   CNT2_BUTTON.DRAW();
   MVC2_BUTTON.DRAW();
   ARM__BUTTON.DRAW();
   BALL_BUTTON.DRAW();
   FILL_BUTTON.DRAW();
   LIT1_BUTTON.DRAW();
   LIT2_BUTTON.DRAW();
   EXIT_BUTTON.DRAW_HILITE(0.7, 0.0, 0.0);
   TARGET_WINDOW.DRAW();
   TARGET_WINDOW.CLEAR();
   glutSwapBuffers();
}

/************************************************************/
/* CALLBACK FUNTIONS.                                       */
/************************************************************/
/****************************************/
/* Mouse control.                       */
/****************************************/
void myMouse(int button, int state, int x, int y) {
   y = (winHeight - y);
   if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
      glFinish();
      if (END1_BUTTON.SELECTED(x, y)) { 
         END1_BUTTON.CLICK();
         end1Flag = 1;
         return;
      }
      if (end1Flag == 1) { 
         cPoints[0][0] = x;
         cPoints[0][1] = y;
         cPoints[0][2] = 0;
         cPoints[0][3] = 1;
         TARGET_DISPLAY();
         end1Flag = 0;
      }
      if (CNT1_BUTTON.SELECTED(x, y)) { 
         CNT1_BUTTON.CLICK();
         cnt1Flag = 1;
         return;
      }
      if (cnt1Flag == 1) { 
         cPoints[1][0] = x;
         cPoints[1][1] = y;
         cPoints[1][2] = 0;
         cPoints[1][3] = 1;
         TARGET_DISPLAY();
         cnt1Flag = 0;
      }
      if (END2_BUTTON.SELECTED(x, y)) { 
         END2_BUTTON.CLICK();
         end2Flag = 1;
         return;
      }
      if (end2Flag == 1) { 
         cPoints[3][0] = x;
         cPoints[3][1] = y;
         cPoints[3][2] = 0;
         cPoints[3][3] = 1;
         TARGET_DISPLAY();
         end2Flag = 0;
      }
      if (CNT2_BUTTON.SELECTED(x, y)) { 
         CNT2_BUTTON.CLICK();
         cnt2Flag = 1;
         return;
      }
      if (cnt2Flag == 1) { 
         cPoints[2][0] = x;
         cPoints[2][1] = y;
         cPoints[2][2] = 0;
         cPoints[2][3] = 1;
         TARGET_DISPLAY();
         cnt2Flag = 0;
      }
      if (ARM__BUTTON.SELECTED(x, y)) { 
         ARM__BUTTON.CLICK();
         if (armWire == 0) armWire = 1;
                      else armWire = 0;
         TARGET_DISPLAY();
      }
      if (FILL_BUTTON.SELECTED(x, y)) { 
         FILL_BUTTON.CLICK();
         if (armSolid == 0) armSolid = 1;
                       else armSolid = 0;
         TARGET_DISPLAY();
      }
      if (BALL_BUTTON.SELECTED(x, y)) { 
         BALL_BUTTON.CLICK();
         if (ballSize ==  5) ballSize = 10;
         else if (ballSize == 10) ballSize = 15;
         else if (ballSize == 15) ballSize = 5;
         TARGET_DISPLAY();
      }

      if (MVE1_BUTTON.SELECTED(x, y)) { 
         MVE1_BUTTON.CLICK();
         if (moveE1 == 0) {
            moveE1 = 1;
            glutIdleFunc(ARM_MOTION);
         } else {
            moveE1 = 0;
            glutIdleFunc(NULL);
         }
      }
      if (MVC1_BUTTON.SELECTED(x, y)) { 
         MVC1_BUTTON.CLICK();
         if (moveC1 == 0) {
            moveC1 = 1;
            glutIdleFunc(ARM_MOTION);
         } else {
            moveC1 = 0;
            glutIdleFunc(NULL);
         }
      }
      if (MVE2_BUTTON.SELECTED(x, y)) { 
         MVE2_BUTTON.CLICK();
         if (moveE2 == 0) {
            moveE2 = 1;
            glutIdleFunc(ARM_MOTION);
         } else {
            moveE2 = 0;
            glutIdleFunc(NULL);
         }
      }
      if (MVC2_BUTTON.SELECTED(x, y)) { 
         MVC2_BUTTON.CLICK();
         if (moveC2 == 0) {
            moveC2 = 1;
            glutIdleFunc(ARM_MOTION);
         } else {
            moveC2 = 0;
            glutIdleFunc(NULL);
         }
      }

      if (LIT1_BUTTON.SELECTED(x, y)) { 
         LIT1_BUTTON.CLICK();
         if (light1 == 0) light1 = 1;
                     else light1 = 0;
         TARGET_DISPLAY();
      }
      if (LIT2_BUTTON.SELECTED(x, y)) { 
         LIT2_BUTTON.CLICK();
         if (light2 == 0) light2 = 1;
                     else light2 = 0;
         TARGET_DISPLAY();
      }
      if (EXIT_BUTTON.SELECTED(x, y)) { 
         EXIT_BUTTON.CLICK();
         glutDestroyWindow(window);
         exit(0);                   
      }
   }
}

/****************************************/
/* Resizes screen (if required)         */
/****************************************/
void myResize(int width, int height) {
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity(); 
   glOrtho(0.0, (GLdouble)winWidth,  0.0, (GLdouble)winHeight, -1000.0, 1000.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

/****************************************/
/* Redisplays screen (if required)      */
/****************************************/
void myDisplay(void) {
   MAIN_WINDOW();
}

/****************************************/
/* Initializes OPENGL parameters    	*/
/****************************************/
void myInit(void) {
   glPointSize(1);
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
   glClear(GL_COLOR_BUFFER_BIT); 
   glMatrixMode(GL_PROJECTION);
   glOrtho(0.0, (GLdouble)winWidth,  0.0, (GLdouble)winHeight, -1000.0, 1000.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();  
}

/************************************************************/
/* MAIN.                                                    */
/************************************************************/
int main(int argc, char **argv) {
   glutInit(&argc, argv) ;
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);  
   glutInitWindowSize(winWidth, winHeight);
   glutInitWindowPosition(winX, winY);
   window = glutCreateWindow("BEZIER ARM by R. Marsh.");
   glutDisplayFunc(myDisplay);
   glutReshapeFunc(myResize);
   glutMouseFunc(myMouse);
   myInit();
   glutMainLoop();
   return 1;
}
